IF  EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[f_GetWeekNumber_GER]') AND xtype in (N'FN', N'IF', N'TF'))
	DROP FUNCTION [dbo].[f_GetWeekNumber_GER]
GO
/*
   31.10.2010 Vers 1.3.00 Olaf Rabbachin (gendert/neu)

   Ermittelt die (fr Deutschland geltende) KW zu einem gegebenen Datum.

*/
CREATE FUNCTION [dbo].[f_GetWeekNumber_GER]
(
   @Date   datetime
)
returns int
AS
/*
Function f_GetWeekNumber_GER returns the
ISO 8601 week of the year for the date passed.

 -> taken from: http://www.sqlteam.com/forums/topic.asp?TOPIC_ID=60510 
    (first post in thread / F_ISO_WEEK_OF_YEAR)
*/
BEGIN
   DECLARE @WeekOfYear int;

   SELECT
      -- Compute week of year as (days since start of year/7)+1
      -- Division by 7 gives whole weeks since start of year.
      -- Adding 1 starts week number at 1, instead of zero.
      @WeekOfYear = (
            datediff(dd
            -- Case finds start of year
            , case 
                  when NextYrStart <= @date then NextYrStart
                  when CurrYrStart <= @date then CurrYrStart
                  else PriorYrStart
               end
            , @date)/7
         ) + 1
   FROM
      (
      SELECT
         -- First day of first week of prior year
         PriorYrStart =
         dateadd(dd,(datediff(dd,-53690,dateadd(yy,-1,aa.Jan4))/7)*7,-53690),
         -- First day of first week of current year
         CurrYrStart =
         dateadd(dd,(datediff(dd,-53690,aa.Jan4)/7)*7,-53690),
         -- First day of first week of next year
         NextYrStart =
         dateadd(dd,(datediff(dd,-53690,dateadd(yy,1,aa.Jan4))/7)*7,-53690)
      FROM
         (
         SELECT
            --Find Jan 4 for the year of the input date
            Jan4   = 
            dateadd(dd,3,dateadd(yy,datediff(yy,0,@date),0))
         ) aa
      ) a;

   return @WeekOfYear;

end
/*

   Test: 2000-2050

   DECLARE @intY int;
   SET @intY=2000;
   WHILE @intY < 2050 BEGIN
      PRINT CAST(@intY AS varchar(4)) + ': 01.01. = ' + CAST(dbo.f_GetWeekNumber_GER(CAST(@intY AS varchar(4)) + '0101') AS varchar) + 
         '; 31.12.: ' + CAST(dbo.f_GetWeekNumber_GER(CAST(@intY AS varchar(4)) + '1231') AS varchar)
      SET @intY=@intY+1
   END

	SELECT dbo.f_GetWeekNumber_GER('20000101') --> soll: 52
	SELECT dbo.f_GetWeekNumber_GER('20080831') --> soll: 35
	SELECT dbo.f_GetWeekNumber_GER('20100101') --> soll: 53
	SELECT dbo.f_GetWeekNumber_GER('20001231') --> soll: 52
   
*/
GO


